package jalview.datamodel.ontology;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * A base class for models of Sequence Ontology and others
 * 
 * @author gmcarstairs
 *
 */
public abstract class OntologyBase implements OntologyI
{
  @Override
  public Set<String> getParentTerms(Set<String> terms)
  {
    Set<String> parents = new HashSet<>(terms);

    boolean childRemoved = true;
    while (childRemoved)
    {
      childRemoved = removeChild(parents);
    }
    return parents;
  }

  /**
   * Removes the first term in the given set found which is a child of another
   * term in the set. Answers true if a child was found and removed, else false.
   * 
   * @param terms
   * @return
   */
  boolean removeChild(Set<String> terms)
  {
    for (String t1 : terms)
    {
      for (String t2 : terms)
      {
        if (t1 != t2)
        {
          if (isA(t1, t2))
          {
            terms.remove(t1);
            return true;
          }
          if (isA(t2, t1))
          {
            terms.remove(t2);
            return true;
          }
        }
      }
    }
    return false;
  }

  @Override
  public List<String> getChildTerms(String parent, List<String> terms)
  {
    List<String> children = new ArrayList<>();
    for (String term : terms)
    {
      if (!term.equals(parent) && isA(term, parent))
      {
        children.add(term);
      }
    }
    return children;
  }
}
